//
// Module: DatafileAttributes
// Author: Richard Saxton
//
// Description:
//
// A class to provide access to keywords from a SAS input file
// and provide sensible defaults if not available.
//
// Modifications: See ChangeLog
//
// $Id: DatafileAttributes.cc,v 1.7 2012/03/23 09:25:00 rsaxton Exp $

/* General */

/* SAS-specific */
#include "Tracer.h"
#include "CalServer.h"
#include "Dal.h"
#include "OalServer.h"
#include "ExceptionErrorHandler.h"

/* Local */
#include "DatafileAttributes.h"

/* Constructor */
DatafileAttributes::DatafileAttributes(const string & datafile):
_datafile(datafile),
_set(0)
{
  PROC;

    /* Open the set */
      _set = dataSetServer->open( datafile, DataSet::Read );

    /* Read the common keywords */

     /* Dates */
      _dateobs = _set->attribute("DATE-OBS")->asString();

      if (_set->hasAttribute("DATE-END"))
      {
         _dateend = _set->attribute("DATE-END")->asString();
      }
      else {
        // No end value so set it to be 10 ks after observation start
         STime numdate(_dateobs);
         numdate = numdate + 10000.0;
         _dateend = numdate.toString();
      }

     /* Pointing */
      _rapnt  = _set->attribute("RA_PNT")->asReal();
      _decpnt  = _set->attribute("DEC_PNT")->asReal();
      _papnt  = _set->attribute("PA_PNT")->asReal();

     /* Observation */
      _telescope = _set->attribute("TELESCOP")->asString();
      _datamode = _set->attribute("DATAMODE")->asString();

    /* Get instrument ID */
      _instrument = _set->attribute("INSTRUME")->asString();

      errstr << "Got instrument "
         << _instrument
         << " which has ID "
         << instrumentID()
         << Sas::message(Msg::App, Msg::Noisy);

     /* Get the observing mode and the EMD params */
      readWindowMode();

     /* Set the instrument name in the CAL */
      calServer.setInstrument(instrumentID());

     /* Set the window mode in the cal */
      calServer.state()->mode()->set(_windowMode);

     /* Get the CCD number */
      if (_set->hasAttribute("CCDNR"))
      {
        _ccdnr = _set->attribute("CCDNR")->asInt();
      } else {

       /* If the CCD number isn't given set it to the central CCD for the
          instrument. This is necessary for timing mode */

         if (instrumentID() == Xmm::EPN) {
            _ccdnr = 4;
         } else { // MOS
            _ccdnr = 1;
         }
      }

      if (_set->hasAttribute("EMDHLOW"))
      {
        _emdhlow= _set->attribute("EMDHLOW")->asInt();
      } else {
        _emdhlow=0;
      }

      if (_set->hasAttribute("EMDHUPP"))
      {
        _emdhupp= _set->attribute("EMDHUPP")->asInt();
      } else {
        _emdhupp=4095;
      }

     /* Filter */
      _filter = _set->attribute("FILTER")->asString();

     /* s/w version */
      if (_set->hasAttribute("PPSVERS"))
      {
        _ppsvers = _set->attribute("PPSVERS")->asString();
      } else {
        _ppsvers = "NotSet";
      }

     /* Get the SRCPOS value if it exists otherwise set to a standard 190 */
      if (_set->hasAttribute("SRCPOS"))
      {
        _SRCPOS = _set->attribute("SRCPOS")->asInt();
      } else {
        _SRCPOS = 190;
      }

     /* Get any attributes from extensions */
      getExtensionAttributes();

     /* Close the file */
      dataSetServer->close(_set);

  ENDPROC;
}

/* Destructor */
DatafileAttributes::~DatafileAttributes()
{
  PROC;
  ENDPROC;
}

/* Method: getExtensionAttributes()
         : Gets attributes from SPECTRUM header if available */
void DatafileAttributes::getExtensionAttributes()
{
  PROC;

   /* Open the spectrum table if it exists */
    Table * spectrumtable;
    if ( _set->hasBlock("SPECTRUM") )
    {
      spectrumtable = _set->table("SPECTRUM");

     /* Get selection expression */
      if (spectrumtable->hasAttribute("SLCTEXPR"))
      {
        _slctexpr = spectrumtable->attribute("SLCTEXPR")->asString();
      }
      else
      {
        errstr
        << "Could not find an attribute SLCTEXPR in table "
        << spectrumtable->qualifiedName()
        << ", so assuming SLCEXPR to be true "
        << Sas::message(Msg::App, Msg::Noisy);

        _slctexpr = "true";
      }
    }

   /* Get the Frame Time keyword from an exposure extension if available.
      Try exposure extension one first and if not available try CCD 2,
      this is for the Imaging part of Timing mode observations */
    Table * exposuretable;
    if ( _set->hasBlock("EXPOSU01") )
    {
      exposuretable = _set->table("EXPOSU01");
    }
    else if ( _set->hasBlock("EXPOSU02") )
    {
      exposuretable = _set->table("EXPOSU02");
    }
    else
    {
      _frameTime = 1.0;
      ALTRETURN;
    }

     /* Get Frame Time in seconds (stored in millisecs) */
    if (exposuretable->hasAttribute("FRMTIME"))
      {
        _frameTime = exposuretable->attribute("FRMTIME")->asInt() / 1000.0;
      }
    else
      {
        _frameTime = 1.0;
        errstr << "Frame Time not found in exposure extension - setting to 1 second"           << Sas::message(Msg::App, Msg::Sparse);
      }

  ENDPROC;
}


const string DatafileAttributes::datafile() const
{
  PROC;
  RETURN _datafile;
}

const string   DatafileAttributes::datamode()   const
{
  PROC;
  RETURN _datamode;
}

const string   DatafileAttributes::observationDate()   const
{
  PROC;
  RETURN _dateobs;
}

const string   DatafileAttributes::observationDateEnd()   const
{
  PROC;
  RETURN _dateend;
}

const string   DatafileAttributes::ppsvers()    const
{
  PROC;
  RETURN _ppsvers;
}

double  DatafileAttributes::rapnt()  const
{
  PROC;
  RETURN _rapnt;
}

double  DatafileAttributes::decpnt() const
{
  PROC;
  RETURN _decpnt;
}

double  DatafileAttributes::papnt()  const
{
  PROC;
  RETURN _papnt;
}

double  DatafileAttributes::frameTime()  const
{
  PROC;
  RETURN _frameTime;
}

const string   DatafileAttributes::filter ()    const
{
  PROC;
  RETURN _filter;
}

void DatafileAttributes::setFilter(const string & filter)
{
  PROC;
    _filter=filter;
  ENDPROC
}

const string   DatafileAttributes::instrument()   const
{
  PROC;
  RETURN _instrument;
}

const string   DatafileAttributes::selectionExpression()   const
{
  PROC;
  RETURN _slctexpr;
}

const string   DatafileAttributes::telescope()   const
{
  PROC;
  RETURN _telescope;
}

const string   DatafileAttributes::observingMode()   const
{
  PROC;
  RETURN _submode;
}
/* AI gcc4 porting
 * error: type qualifiers ignored on function return type
 * leading const removed
 */
int DatafileAttributes::emdhlow()   const
{
  PROC;
  RETURN _emdhlow;
}
/* AI gcc4 porting
 * error: type qualifiers ignored on function return type
 * leading const removed
 */
int DatafileAttributes::emdhupp()   const
{
  PROC;
  RETURN _emdhupp;
}

/* Method to find the observing window mode */
void DatafileAttributes::readWindowMode()
{
  PROC;

     /* Read the observing mode from the SUBMODE keyword */
      if (_set->hasAttribute("SUBMODE"))
      {
        _submode = _set->attribute("SUBMODE")->asString();
      } else {
        _submode = " ";
      }

     /* Read the data collection mode from the DATAMODE keyword
      string datamode=" ";
      if (_set->hasAttribute("DATAMODE"))
      {
        datamode = _set->attribute("DATAMODE")->asString();
      }
     */
      errstr<<"Data mode: "<<_datamode
            << Sas::message(Msg::App, Msg::Verbose);

     /* If the observing mode is timing and the datamode is imaging
        then the spectrum has been taken from the outer chips of
        a timing mode obs which are actually in imagind mode */
      if (_datamode=="IMAGING" &&
           (_submode=="FastUncompressed" || _submode=="FastCompressed") )
               { _submode= "PrimeFullWindow"; }

     /* Check if submode string is ok and if not set to default */
      ExceptionErrorHandler local;
      ErrorHandler* oldHandler = errHandler.handler(&local);
      try {
            _windowMode = Xmm::toMode(_submode);
      } catch(SasException::Error) { errstr<<"SUBMODE not valid or missing "<<
          "- using FullWindow" <<Sas::warning("UnknownModeString");
        _submode = "PrimeFullWindow";
        _windowMode = Xmm::toMode(_submode); // d/f
      }
      errHandler.handler(oldHandler);

      errstr<<"Window mode: "<<_windowMode
            << Sas::message(Msg::App, Msg::Verbose);
}

Xmm::Mode DatafileAttributes::getWindowMode() const
{
  PROC;
  RETURN _windowMode;
}

Xmm::Instrument DatafileAttributes::instrumentID()  const
{
  PROC;
  RETURN Xmm::toInstrument(_instrument);
}
/* AI gcc4 porting
 * error: type qualifiers ignored on function return type
 * leading const removed
 */
int DatafileAttributes::ccdNr() const
{
  PROC;
  RETURN _ccdnr;
}

int DatafileAttributes::spec_SRCPOS() const
{
  PROC;
  RETURN _SRCPOS;
}

/* Method: intrinsicFrameTime()
  Purpose: Returns the frame time for this instrument/obs. mode
*/
double DatafileAttributes::intrinsicFrameTime() const
{
  PROC;

    double timebin=0.0;

   /* Set the frame time width depending on the instrument and
      observing mode */

    switch(instrumentID())
    {
      case Xmm::EPN:
       {
         switch(getWindowMode())
         {
           case Xmm::PRIME_FULL_WINDOW:
            {
             timebin = 0.07336;
             break;
            }
           case Xmm::PRIME_FULL_WINDOW_EXTENDED:
            {
             timebin = 0.1992;
             break;
            }
           case Xmm::PRIME_SMALL_WINDOW:
            {
             timebin = 0.0057;
             break;
            }
           case Xmm::PRIME_LARGE_WINDOW:
            {
             timebin = 0.0477;
             break;
            }
           case Xmm::FAST_TIMING:
            {
             timebin = 0.00003;
             break;
            }
           case Xmm::FAST_BURST:
            {
             timebin = 0.000007;
             break;
            }
           default:
             errstr << "Mode unknown - setting time bin size to 1 second"
                    << Sas::warning("UnknownModeString");
             timebin = 1.0;
          } // end obs. mode switch
         break;
       } // end - PN case

      case Xmm::EMOS1 :
      case Xmm::EMOS2 :
       {
         /* Use the Frame time from the event file */
          timebin = frameTime();
          break;
       } // end - MOS case
      default:
          errstr << "Instrument unknown"
                 << Sas::error("invalidInstrument");
      }


     RETURN timebin;

} // end proc
