//
// Module: RectanglePsfModel
// Author: Richard Saxton
//
// Description: 
//   Rectangular PSF template - not currently used.
//
// Sets up a PSF model for a circular source box.
// It is valid for a circular source box centred on the source.
//
// Modifications: See ChangeLog
// 
// $Id: RectanglePsfModel.cc,v 1.2 2008/09/10 09:13:54 rsaxton Exp $

// General classes
#include <iostream>
#include <cmath>

// SAS-specific classes

// General utility functions
#include "errstr.h"
#include "Tracer.h"
#include "NumericalRange.h"
#include "ArfDataServer.h"
#include "EncircledEnergyDataServer.h"

// Local classes
#include "RectanglePsfModel.h"

/* Constructor */
RectanglePsfModel::RectanglePsfModel( double Energy,    
		    double Theta,                
		    double Phi,                 
		    vector<double> HalfWidth,             
		    unsigned int NGridEls     
		    ):
  CirclePsfModel(Energy,Theta,Phi,HalfWidth,NGridEls),
  _theta(Theta),
  _phi(Phi),
  _ngridels(NGridEls)
{
  PROC;

  /* Find the minor and major axes and the diagonal length */
      if ( HalfWidth[0] < HalfWidth[1] ) {
         _radii[0]=HalfWidth[0];
         _radii[1]=HalfWidth[1];
      } else {
         _radii[0]=HalfWidth[1];
         _radii[1]=HalfWidth[0];
      }

   _diagonal = sqrt(_radii[0] * _radii[0] + _radii[1] * _radii[1]);

   errstr 
    << "RectanglePsfModel: constructed with "
    << " Energy " << _energy
    << " Theta " << Theta 
    << " Phi " << Phi 
    << " Radius " << HalfWidth[0]
    << " NgridEls " << NGridEls
    << Sas::message(Msg::App, Msg::Verbose);


  ENDPROC;
};

RectanglePsfModel::~RectanglePsfModel()
{
  PROC;
  errstr 
    << "RectanglePsfModel destroyed."
    << Sas::message(Msg::App, Msg::Verbose);

  ENDPROC;

}

/* Method: rad_psf() - Calculate integrated PSF over the rectangle. 
   This uses the circular PSF (EXTENDED mode) to calculate the
   total EEF within a rectangular area */

double RectanglePsfModel::rad_psf() const
{
  PROC;

  /* Find the encircled energy for the circle described by the minor axis */
      errstr
        << "Getting Encircled energy (energy): "<<_energy
        << "radius "<<_radii[0]
        << "Ngridels "<<_ngridels
        << Sas::message(Msg::App, Msg::Verbose);

      double ee_inner = (*eef)(_radii[0]) / _ngridels;

  /* Find the encircled energy for the circle touched by the corners of
     the rectangle */

      double ee_outer = (*eef)(_diagonal) / _ngridels;

  /* Calculate the fraction of the annulus between these inner and outer
     circles which is contained within the rectangle */

      double afrac = ( 4.0 * _radii[0] * _radii[1] -
                         M_PI * _radii[0] * _radii[0] ) /
                     ( M_PI * _diagonal * _diagonal -
                         M_PI * _radii[0] * _radii[0] ) /

  /* Compute the total EEF */

      double ee = ee_inner + (ee_outer - ee_inner) * afrac;

  RETURN ee;
}

/* Method to set the mode */
string RectanglePsfModel::mode() const
{
  PROC; 

  RETURN "RECTANGLE";
}
